# Convert trs file into human-readable report

import sys
import gff

#########################################
# Command line
#########################################

FileName = sys.argv[1]

#########################################
# Function definitions
#########################################

def Quit(s):
	print >> sys.stderr, "*** ERROR ***", sys.argv
	print >> sys.stderr, "*************", s
	sys.exit(1)

def DoRec():
	global Fams
	global SuperFams

	if gff.Feature != "trs":
		return

	gff.ParseTRSAttrs()

	Labels.append(gff.Label)
	Starts.append(gff.Start)
	Ends.append(gff.End)
	Strands.append(gff.Strand)
	Piles.append(gff.TRSPile)
	Fams.append(gff.TRSFam)
	SuperFams.append(gff.TRSSuperFam)
	
	if gff.HasAnnot():
		gff.ParseAnnotAttrs()
		Annots.append(gff.AnnotString)
	else:
		Annots.append("")
	
def PrintSuperFamHeader(SuperFam):
	print ""
	print "================================"
	print "Superfamily %d" % SuperFam
	print "================================"

def PrintFamHeader(SuperFam, Fam):
	print
	print "Family %d.%d" % (SuperFam, Fam)
	print "        Sequence       Start         End  +  Length      Pile  Annotation"
	print "================  ==========  ==========  =  ======  ========  =========="

def PrintFamFooter(FamCount, TotalFamLength):
	print "================                             ======"
	print "    %6d piles                             %6d" % (FamCount, TotalFamLength/FamCount)
	
def ByFam(i, j):
	Fam_i = Fams[i]
	Fam_j = Fams[j]
	SuperFam_i = SuperFams[i]
	SuperFam_j = SuperFams[j]
	if SuperFam_i == SuperFam_j:
		return Fam_i - Fam_j
	return SuperFam_i - SuperFam_j

#########################################
# Main program
#########################################

Labels = []
Starts = []
Ends = []
Strands = []
Piles = []
Fams = []
SuperFams = []
Annots = []

gff.GetRecs(FileName, DoRec)

RecordCount = len(Fams)
Order = range(0, RecordCount)
Order.sort(ByFam)

CurrentFam = -1
CurrentSuperFam = -1
FamCount = 0
FamTotalLength = 0
for i in Order:
	if Fams[i] != CurrentFam and FamCount > 0:
		PrintFamFooter(FamCount, FamTotalLength)
	if SuperFams[i] != CurrentSuperFam:
		CurrentSuperFam = SuperFams[i]
		PrintSuperFamHeader(CurrentSuperFam)
	if Fams[i] != CurrentFam:
		FamCount = 0
		FamTotalLength = 0
		CurrentFam = Fams[i]
		PrintFamHeader(CurrentSuperFam, CurrentFam)
		CurrentFam = Fams[i]
	
	Label = Labels[i]
	Start = Starts[i]
	End = Ends[i]
	Strand = Strands[i]
	Pile = Piles[i]
	Annot = Annots[i]
	Fam = Fams[i]
	SuperFam = SuperFams[i]
	
	Length = End - Start + 1
	
	print "%16.16s  %10d  %10d  %c  %6d  %8d  %s" % (Label, Start, End, Strand, Length, Pile, Annot)

	FamCount += 1
	FamTotalLength += Length

if FamCount > 0:
	PrintFamFooter(FamCount, FamTotalLength)
